@@ -4,7 +4,7 @@ use strict;
 use warnings;
 
 use lib qw{blib/arch blib/lib};
-use Variable::Magic qw/wizard getsig cast/;
+use Variable::Magic qw/wizard cast/;
 use Tie::Hash;
 
 my $wiz = wizard copy => sub { print STDERR "COPY $_[2] => $_[3]\n" },
